//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "www.forex-tsd,com"

#property indicator_separate_window
#property  indicator_buffers 11
#property indicator_color1   LimeGreen
#property indicator_color2   LimeGreen
#property indicator_color3   PaleVioletRed
#property indicator_color4   PaleVioletRed
#property indicator_color5   Green
#property indicator_color6   Tomato
#property indicator_color7   DeepSkyBlue
#property indicator_color8   Red
#property indicator_color9   Red
#property indicator_color10  Orange
#property indicator_color11  DimGray
#property indicator_width1   3 
#property indicator_width3   3 
#property indicator_width7   2 
#property indicator_width8   2 
#property indicator_width9   2 
#property indicator_width10  2 
#property indicator_style11  STYLE_DOT

//
//
//
//
//

extern string TimeFrame              = "Current time frame";
extern int    DMILength              = 14;
extern int    Level                  = 20;
extern bool   alertsOn               = false;
extern bool   alertsOnCurrent        = true;
extern bool   alertsMessage          = true;
extern bool   alertsSound            = false;
extern bool   alertsNotify           = false;
extern bool   alertsEmail            = false;
extern string soundFile              = "alert2.wav";
extern bool   ShowArrows             = false;
extern string arrowsIdentifier       = "DMI Arrows1";
extern double arrowsUpperGap         = 1.0;
extern double arrowsLowerGap         = 1.0;
extern color  arrowsUpColor          = LimeGreen;
extern color  arrowsDnColor          = Red;
extern int    arrowsUpCode           = 241;
extern int    arrowsDnCode           = 242;
extern bool   Interpolate            = true;

extern bool   ShowADX                = true;
extern bool   ShowADXR               = false;
extern bool   ShowDIPlusDIMinusHisto = true;
extern bool   ShowDIPlusDIMinusLines = true; 
//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double adxDa[];
double adxDb[];
double ADXR[];
double ADXLevel[];
double dihup[];
double dihum[];
double dihdp[];
double dihdm[];
double trend[];
double slope[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(13);
   SetIndexBuffer(0, dihup);     
   SetIndexBuffer(1, dihdp);    
   SetIndexBuffer(2, dihdm);     
   SetIndexBuffer(3, dihum);    
   SetIndexBuffer(4, DIp);      
   SetIndexBuffer(5, DIm);      
   SetIndexBuffer(6, ADX);  
   SetIndexBuffer(7, adxDa); 
   SetIndexBuffer(8, adxDb);    
   SetIndexBuffer(9, ADXR);     
   SetIndexBuffer(10,ADXLevel); SetIndexLabel(10,NULL);
   SetIndexBuffer(11,trend);
   SetIndexBuffer(12,slope);
      for (int i=0;i<13;i++) SetIndexEmptyValue(i,0.00);
   if (ShowDIPlusDIMinusHisto)
   {
      SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"DI+");
      SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"DI-");  
      SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"DI+");
      SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"DI-");  
   }
   else
   {
      SetIndexStyle(0,DRAW_NONE); 
      SetIndexStyle(1,DRAW_NONE); 
      SetIndexStyle(2,DRAW_NONE); 
      SetIndexStyle(3,DRAW_NONE); 
   }
   
   if (ShowDIPlusDIMinusLines)
   {
      SetIndexStyle(4,DRAW_LINE); SetIndexLabel(4,"DI+");
      SetIndexStyle(5,DRAW_LINE); SetIndexLabel(5,"DI-");  
   }
   else
   {
      SetIndexStyle(4,DRAW_NONE); 
      SetIndexStyle(5,DRAW_NONE); 
   }
   
   if (ShowADX)
   {
      SetIndexStyle(6,DRAW_LINE); SetIndexLabel(6,"ADX"); 
      SetIndexStyle(7,DRAW_LINE); SetIndexLabel(7,"ADX"); 
      SetIndexStyle(8,DRAW_LINE); SetIndexLabel(8,"ADX"); 
   }
   else
   {
      SetIndexStyle(6,DRAW_NONE);  
      SetIndexStyle(7,DRAW_NONE);  
      SetIndexStyle(8,DRAW_NONE);  
   }
   
   if (ShowADXR)
   {
      SetIndexStyle(9,DRAW_LINE); SetIndexLabel(9,"ADXR"); 
   }
   else
   {
      SetIndexStyle(9,DRAW_NONE);  
   }
      
   //
   //
   //
   //
   //

      indicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" Wilder\'s DMI ("+DMILength+")");
   return(0);
}

int deinit() 
{  
   deleteArrows(); 
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][3];
#define _DIp 0
#define _DIm 1
#define _TR  2

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (returnBars)  { dihup[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      if (slope[limit]==-1) CleanPoint(limit,adxDa,adxDb);
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double sf = (DMILength-1.0)/DMILength;
      for (i=limit, r=Bars-i-1;i>=0;i--,r++)
      {
         double currTR  = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         double DeltaHi = High[i] - High[i+1];
	      double DeltaLo = Low[i+1] - Low[i];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            work[r][_DIp] = sf*work[r-1][_DIp] + plusDM;
            work[r][_DIm] = sf*work[r-1][_DIm] + minusDM;
            work[r][_TR]  = sf*work[r-1][_TR]  + currTR;
            ADXLevel[i]   = Level;

         //
         //
         //
         //
         //
                  
            DIp[i]   = 0.00;                   
            DIm[i]   = 0.00;                   
            dihup[i] = 0.00;
            dihdp[i] = 0.00;
            dihdm[i] = 0.00;
            dihum[i] = 0.00;
            if (work[r][_TR] > 0)
               {              
                  DIp[i] = 100.00 * work[r][_DIp]/work[r][_TR];
                  DIm[i] = 100.00 * work[r][_DIm]/work[r][_TR];
               }            
               if (DIp[i]>DIm[i] && DIp[i]>DIp[i+1]) dihup[i] = DIp[i]-DIm[i];
               if (DIp[i]>DIm[i] && DIp[i]<DIp[i+1]) dihdp[i] = DIp[i]-DIm[i];
               if (DIp[i]<DIm[i] && DIm[i]>DIm[i+1]) dihdm[i] = DIp[i]-DIm[i];
               if (DIp[i]<DIm[i] && DIm[i]<DIm[i+1]) dihum[i] = DIp[i]-DIm[i];
               
                  double DX;
                  if((DIp[i] + DIm[i])>0) 
                       DX = 100*MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                  else DX = 0.00;
                  ADX[i]  = sf*ADX[i+1] + DX/DMILength; 
                  ADXR[i] = 0.5*(ADX[i] + ADX[i+DMILength]);
               
               //
               //
               //
               //
               //
               
               adxDa[i] = 0.00;
               adxDb[i] = 0.00;
               slope[i] = slope[i+1];
               trend[i] = trend[i+1];
                  if (ADX[i]>ADX[i+1]) slope[i] = 1;
                  if (ADX[i]<ADX[i+1]) slope[i] =-1;
                  if (DIp[i]>DIm[i])   trend[i] = 1;
                  if (DIp[i]<DIm[i])   trend[i] =-1;
                  if (slope[i]==-1)    PlotPoint(i,adxDa,adxDb,ADX); 
               
                  //
                  //
                  //
                  //
                  //
     
                  if (ShowArrows)
                  {
                    deleteArrow(Time[i]);
                    if (trend[i] != trend[i+1])
                    {
                      if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
                      if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
                    }
                  }
      }
      
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
         if (alertsOnCurrent)
              int whichBar = 0;
         else     whichBar = 1; 
         if (trend[whichBar] != trend[whichBar+1])
         if (trend[whichBar] == 1)
               doAlert("crossed up");
         else  doAlert("crossed down");       
   }        
      
      
   return(0);      
   }
      
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (slope[limit]==-1) CleanPoint(limit,adxDa,adxDb);
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         ADXLevel[i] = Level;
         dihup[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,0, y);
         dihdp[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,1, y);
         dihdm[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,2, y);
         dihum[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,3, y);
         DIp[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,4, y);
         DIm[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,5, y);
         ADX[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,6, y);
         adxDa[i] = 0.00;
         adxDb[i] = 0.00;
         ADXR[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,9, y);
         slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",DMILength,Level,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,12,y);
         
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               DIp[i+k]  = DIp[i]  + (DIp[i+n] -DIp[i]) *k/n;
               DIm[i+k]  = DIm[i]  + (DIm[i+n] -DIm[i]) *k/n;
               ADX[i+k]  = ADX[i]  + (ADX[i+n] -ADX[i]) *k/n;
               ADXR[i+k] = ADXR[i] + (ADXR[i+n]-ADXR[i])*k/n;
            }
         
   } 
   for (i=limit;i>=0;i--) if (slope[i]==-1) PlotPoint(i,adxDa,adxDb,ADX);
   return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != 0.00) && (second[i+1] != 0.00))
        second[i+1] = 0.00;
   else
      if ((first[i] != 0.00) && (first[i+1] != 0.00) && (first[i+2] == 0.00))
          first[i+1] = 0.00;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == 0.00)
      {
         if (first[i+2] == 0.00) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = 0.00;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = 0.00;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = 0.00;
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
} 

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Wilders DMI ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Wilders DMI "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}



